package org.telosys.tools.repository;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.telosys.tools.commons.TelosysToolsLogger;
import org.telosys.tools.commons.TelosysToolsException;
import org.telosys.tools.commons.config.ClassNameProvider;
import org.telosys.tools.db.model.DatabaseColumn;
import org.telosys.tools.db.model.DatabaseForeignKey;
import org.telosys.tools.db.model.DatabaseModelManager;
import org.telosys.tools.db.model.DatabaseTable;
import org.telosys.tools.db.model.DatabaseTables;
import org.telosys.tools.repository.config.InitializerChecker;
import org.telosys.tools.repository.model.Column;
import org.telosys.tools.repository.model.Entity;
import org.telosys.tools.repository.model.ForeignKey;
import org.telosys.tools.repository.model.RepositoryModel;

/**
 * @author Sylvain LEROY, Laurent GUERIN, Eric LEMELIN
 * 
 */

public class RepositoryUpdator extends RepositoryManager
{
	private UpdateLogWriter _updateLogger = null;

	/**
	 * Constructor
	 * 
	 * @param inichk
	 * @param classNameProvider
	 * @param logger
	 */
	public RepositoryUpdator(InitializerChecker inichk, ClassNameProvider classNameProvider, 
			TelosysToolsLogger logger, UpdateLogWriter updateLogger) 
	{
		super(inichk, classNameProvider, logger);
		_updateLogger = updateLogger;
	}

	//private Column addEntityAttribute(Entity entity, ColumnMetaData col, List listPK) 
	private Column addEntityAttribute(Entity entity, DatabaseColumn dbColumn ) 
	{
		/***
		//--- Column Name
		String dbColName = col.getName();
		if (dbColName == null)
			dbColName = "";

		//--- Column JDBC Type (cf "java.sql.Types" )
		int iDbTypeCode = col.getTypeCode();

		//--- Column Type (original database type)
		String dbTypeName = col.getTypeName();
		if (dbTypeName == null)
			dbTypeName = "";

		//--- Column NOT NULL ( "true" or "false" )
		String dbNotNull = col.getNotNull();

		//--- Column Size : For char or date types this is the maximum number
		// of characters,
		// for numeric or decimal types this is precision
		int iDbSize = col.getSize();

		//--- Java field name and type
		String sJavaType = "???";
		String sJavaName = "???";
		String sLongTextFlag = null;
		String sDateType = null;
		try {
			sJavaType = _inichk.getAttributeType(dbTypeName, iDbTypeCode);
			if (sJavaType == null)
				sJavaType = "null";

			sJavaName = _inichk.getAttributeName(dbColName, dbTypeName, iDbTypeCode);
			if (sJavaName == null)
				sJavaName = "null";

			// --- Special informations
			sLongTextFlag = _inichk.getAttributeLongTextFlag(dbTypeName, iDbTypeCode, sJavaType);
			sDateType = _inichk.getAttributeDateType(dbTypeName, iDbTypeCode, sJavaType);

		} catch (Throwable t) {
			_logger.log("   ERROR : " + t.toString() + " - " + t.getMessage());
		}
		_logger.log("   - Column : " + dbColName + " ( " + iDbTypeCode + " : " + dbTypeName + " ) ---> " + sJavaName
				+ " ( " + sJavaType + " ) ");

		if (sJavaName.equals("boolean")) {
			// bUseBooleanType = true;
		}

		// --- Create XML element "column"
		Column column = new Column();
		column.setDatabaseName(dbColName);
		column.setDatabaseTypeName(dbTypeName);
		column.setDatabaseTypeCode(iDbTypeCode);
		column.setNotNull(dbNotNull);
		column.setDatabaseSize(iDbSize);
		column.setJavaName(sJavaName);
		column.setJavaType(sJavaType);
		
		if (sLongTextFlag != null) {
			column.setLongText(sLongTextFlag);
		}
		if (sDateType != null) {
			column.setDateType(sDateType);
		}

		// --- If this column is in the Table Primary Key
		if (listPK.contains(dbColName.toUpperCase())) {
			column.setPrimaryKey(true);
		}
		***/
		
		Column column = buildColumn( dbColumn ) ;
		
		//--- Add the "column" to the "entity"
		entity.storeColumn(column);
		return column;
	}

//	private int updateEntityAttribute(Column column, ColumnMetaData col, List listPK) 
	private int updateEntityAttribute(Column column, DatabaseColumn col) 
	{
		int r = 0;

//		// --- Column Name
//		//String dbColName = col.getName();
//		String dbColName = col.getColumnName();
//		if (dbColName == null)
//			dbColName = "";

		//--- Update the column 
		r = r + updateDbType(column, col.getDbTypeName()); // Database native type		
		r = r + updateTypeCode(column, col.getJdbcTypeCode()); // JDBC type code 
		r = r + updateNotNull(column, col.getNotNullAsString()); // Not null
		r = r + updateSize(column, col.getSize()); // Size

//		// --- If this column is in the Table Primary Key
//		boolean isPrimaryKey = listPK.contains(dbColName.toUpperCase());
//		r = r + updatePrimaryKey(column, isPrimaryKey);
		r = r + updatePrimaryKey(column, col.isInPrimaryKey()); // Column in Primary Key
		
		// TODO 
		// . default value
		// . comment
		// . auto incremented
		// . in foreign key

		return r;
	}

	private int updateTypeCode( Column column, int iDbTypeCode) 
	{
		int r = 0;
		int i = column.getJdbcTypeCode();
		if ( i != iDbTypeCode ) {
			_updateLogger.println(" . Column '" + column.getDatabaseName() + "' : JDBC type code changed to " + iDbTypeCode);
			column.setJdbcTypeCode(iDbTypeCode);
			r++;
		}
		return r;
	}

	private int updateDbType( Column column, String sDbType ) 
	{
		int r = 0;
		String s = column.getDatabaseTypeName();
		if ( ! s.equals(sDbType) ) {
			_updateLogger.println(" . Column '" + column.getDatabaseName() + "' : Database type changed to " + sDbType);
			column.setDatabaseTypeName(sDbType);
			r++;
		}
		return r;
	}

	private int updateNotNull( Column column, String sNotNull) 
	{
		int r = 0;
		String s = column.getDatabaseNotNullAsString();
		if ( ! s.equals(sNotNull) ) {
			_updateLogger.println(" . Column '" + column.getDatabaseName() + "' : NotNull changed to " + sNotNull);
			column.setDatabaseNotNull(sNotNull);
			r++;
		}
		return r;
	}

	private int updateSize( Column column, int iSize) 
	{
		int r = 0;
		if ( column.getDatabaseSize() != iSize ) 
		{
			_updateLogger.println(" . Column '" + column.getDatabaseName() + "' : Size changed to " + iSize);
			column.setDatabaseSize(iSize);
			r++;
		}
		return r;
	}

	private int updatePrimaryKey( Column column, boolean isPrimaryKey) 
	{
		int r = 0;
		if ( column.isPrimaryKey() != isPrimaryKey )
		{
			_updateLogger.println(" . Column '" + column.getDatabaseName() + "' : Primary Key flag changed to " + isPrimaryKey);
			column.setPrimaryKey(isPrimaryKey);
			r++;
		}
		return r;
	}

	// -----------------------------------------------------------------------------------------------------
	// UPDATE REPOSITORY
	// -----------------------------------------------------------------------------------------------------
	/**
	 * Updates the given repository with the database metadata.
	 * 
	 * @param con
	 * @param repositoryModel
	 * @param sCatalog
	 * @param sSchema
	 * @param sTableNamePattern
	 * @param arrayTableTypes
	 * @return the number of changes
	 * @throws TelosysToolsException
	 */
	public int updateRepository(Connection con, RepositoryModel repositoryModel, String sCatalog,
			String sSchema, String sTableNamePattern, String[] arrayTableTypes) throws TelosysToolsException 
	{
		int changesCount = 0 ;
		//_logger.log("--> Update repository ( file = '" + file.getFullPath() + "' )");

		Date now = new Date();

		try {
			_logger.log(" . get meta-data ");
			//DatabaseMetaData dbmd = getMetaData(con);

			try {

				_logger.log(" . update repository from database tables");
				_updateLogger.println("Update date : " + now);
				
				//--- Load the Database Model
				DatabaseModelManager manager = new DatabaseModelManager( this.getLogger() );
				DatabaseTables dbTables = manager.getDatabaseTables(con, sCatalog, sSchema, sTableNamePattern, arrayTableTypes);

				//updateRepository(repositoryModel, dbmd, sCatalog, sSchema, sTableNamePattern, arrayTableTypes);
				changesCount = updateRepository(repositoryModel, dbTables);

			} catch (SQLException e) {
				e.printStackTrace();
				throw new TelosysToolsException("SQLException", e);
			} catch (Throwable t) {
				t.printStackTrace();
				throw new TelosysToolsException("Exception", t);
			}
		}
		finally {
			_updateLogger.close();
		}
		return changesCount ;
	}

	private int updateRepository(RepositoryModel repositoryModel, DatabaseTables dbTables ) throws SQLException 
	{
//		if (sTableNamePattern == null) {
//			sTableNamePattern = "%";
//		}
		int changesCount = 0 ;
		
		LinkedList<String> databaseTables = new LinkedList<String>();

		// --- Get tables list
		//ResultSet rsTables = dbmd.getTables(sCatalog, sSchema, sTableNamePattern, arrayTableTypes);
		
		// -----------------------------------------------------------------------
		// STEP 1 : Update existing tables and Create new ones
		// -----------------------------------------------------------------------
		// --- For each table in the database ...
		int iTablesCount = 0;
		//while (rsTables.next()) 
		Iterator iter = dbTables.iterator();
		while ( iter.hasNext() )
		{
			iTablesCount++;
			DatabaseTable dbTable = (DatabaseTable) iter.next();
			
//			String sCurrentCatalog = rsTables.getString(1);
//			String sCurrentSchema = rsTables.getString(2);
//			String sTableName = rsTables.getString(3);
			_logger.log("   --------------------------------------------------------------");
			_logger.log("   Table '" + dbTable.getTableName() 
					+ "' ( catalog = '" + dbTable.getCatalogName() 
					+ "', schema = '"+ dbTable.getSchemaName() + "' )");

			String sTableName = dbTable.getTableName();

			// --- Store the table name in the list used to delete entities
			databaseTables.add(sTableName);

			_updateLogger.println(" ");
			Entity entity = repositoryModel.getEntityByName(sTableName);
			
			if ( entity != null ) 
			{
				// --- The <table ..> exists in the repository => update it
				_updateLogger.println(" Table '" + sTableName + "' found in repository");
				//int r = updateEntity(repositoryModel, dbmd, sCurrentCatalog, sCurrentSchema, sTableName, entity);
				int entityChanges = updateEntity(repositoryModel, dbTable, entity);
				if (entityChanges > 0) {
					_updateLogger.println(" (*) table '" + sTableName + "' updated : " + entityChanges + " change(s)");
				} else {
					_updateLogger.println(" (=) table '" + sTableName + "' unchanged");
				}
				changesCount = changesCount + entityChanges ;
			} else {
				// --- The TABLE doesn't exist in the repository => add it
				_updateLogger.println(" Table '" + sTableName + "' not found in repository");
				//addEntity(repositoryModel, dbmd, sCurrentCatalog, sCurrentSchema, sTableName);
				addEntity(repositoryModel, dbTable) ;
				_updateLogger.println(" (+) table '" + sTableName + "' added");
				changesCount++;
			}
		}
		//rsTables.close();

		// -----------------------------------------------------------------------
		// STEP 2 : Remove tables that no longer exist in the database
		// -----------------------------------------------------------------------
		// --- For each table in the repository ...
		String[] tableNames = repositoryModel.getEntitiesNames();
		for (int i = 0; i < tableNames.length; i++) 
		{
			String sTableName = tableNames[i];
			if (checkTableExistsInDatabase(sTableName, databaseTables) != true) {
				//--- This table in the repository no longer exists in the database
				_updateLogger.println(" ");
				_updateLogger.println(" Table '" + sTableName + "' no longer exists in database");
				//--- => Remove it
				repositoryModel.removeEntity(sTableName);
				_updateLogger.println(" (-) table '" + sTableName + "' removed");
				changesCount++;
			}
		}
		
		return changesCount ;
	}
	
	private int updateEntity( RepositoryModel repositoryModel, DatabaseTable dbTable, Entity entity) throws SQLException 
	{
		int changeCount = 0;

//		List listColumns = getColumnsMetaData(dbmd, sCatalog, sSchema, sTableName);

		//--- Get the PrimaryKey columns list
//		List listPK = getColumnsPK(dbmd, sCatalog, sSchema, sTableName);

		//--- Get the ForeignKeys list
//		List listFK = getForeignKeyParts(dbmd, sCatalog, sSchema, sTableName);

		//--------------------------------------------------------------------------------
		// 1) remove the colums that doesn't exist in the Database 
		//--------------------------------------------------------------------------------
		//--- For each column in the repository ...
		Column[] columns = entity.getColumns();
		for (int i = 0; i < columns.length; i++) 
		{
			Column e = columns[i];
			String sColumnName = e.getDatabaseName();
			// Does it still exist in the DATABASE ?
			//if (checkColumnExistence(sColumnName, listColumns) != true)
			if ( null == dbTable.getColumnByName(sColumnName) )
			{
				//--- This column doesn't exist in the DB => remove it from the repo
				entity.removeColumn(e);
				changeCount++;
				_updateLogger.println(" . Column '" + sColumnName + "' removed");
			}
		}

		//--------------------------------------------------------------------------------
		// 2) remove the foreign keys that doesn't exist in the Database 
		//--------------------------------------------------------------------------------
		//--- For each fk in the repository ...
		ForeignKey[] fkeys = entity.getForeignKeys();
		for (int i = 0; i < fkeys.length; i++) 
		{
			ForeignKey fk = fkeys[i];
			String sFkName = fk.getName();
			// Does it still exist in the DATABASE ?
			//if (!checkFkExistence(sFkName, listFK))
			if ( null == dbTable.getForeignKeyByName(sFkName) )
			{
				//--- This FK doesn't exist in the DB => remove it from the repo
				entity.removeForeignKey(fk);
				changeCount++;
				_updateLogger.println(" . Foreign key '" + sFkName + "' removed");
			}
		}

		//--------------------------------------------------------------------------------
		// 3) UPDATE existing COLUMNS if necessary and ADD new ones
		//--------------------------------------------------------------------------------
		//--- For each column of the table in the DataBase ...
//		int n = listColumns.size();
//		for (int i = 0 ; i < n ; i++) 
		List dbColumns = dbTable.getColumns();
		Iterator iter = dbColumns.iterator();
		while ( iter.hasNext() )
		{
//			ColumnMetaData columnMD = (ColumnMetaData) listColumns.get(i);
			DatabaseColumn dbColumn = (DatabaseColumn) iter.next();
//			String sColumnName = columnMD.getName();
			String sColumnName = dbColumn.getColumnName();
			
			//--- Search this column in the REPOSITORY
			Column column = entity.getColumn(sColumnName);
			if ( column != null ) 
			{
				// The column exists => update it
				//changeCount = changeCount + updateEntityAttribute(column, columnMD, listPK);
				changeCount = changeCount + updateEntityAttribute(column, dbColumn);

			} else {
				// The column doesn't exist => add it
				//column = addEntityAttribute(entity, columnMD, listPK);
				column = addEntityAttribute(entity, dbColumn);
				_updateLogger.println(" . Column '" + sColumnName + "' added");
				changeCount++;
			}
			//--- If this column is a member of a Foreign Key
			//setFkAttribute(sColumnName, column, listFK);
		}

		//--------------------------------------------------------------------------------
		// 4) UPDATE existing FOREIGN KEYS if necessary and ADD new ones
		//--------------------------------------------------------------------------------
		//--- For each FK of the table in the DataBase ...( v 0.9.0 )
		//List newFk = new ArrayList();
		//for (int i = 0 ; i < listFK.size() ; i++) 
		List dbForeignKeys = dbTable.getForeignKeys();
		Iterator iterDbForeignKeys = dbForeignKeys.iterator();
		while ( iterDbForeignKeys.hasNext() )
		{
			//ForeignKeyMetaData fkMD = (ForeignKeyMetaData) listFK.get(i);
			DatabaseForeignKey dbForeignKey = (DatabaseForeignKey) iterDbForeignKeys.next();
			
			//String sFkName = fkMD.getFkName();
			String sFkName = dbForeignKey.getForeignKeyName();
			
			//--- Search this foreign key in the REPOSITORY
			ForeignKey newForeignKey = buildForeignKey(dbForeignKey) ;
			ForeignKey foreignKey = entity.getForeignKey(sFkName);
			if ( foreignKey != null ) 
			{
				// The FK exists => update it if it has changed
				if ( ! foreignKey.equals( newForeignKey ) )
				{
					// 
					entity.storeForeignKey(newForeignKey);
					changeCount++;
					_updateLogger.println(" . Foreign key '" + sFkName + "' updated");
				}
			}
			else
			{
				// The FK doesn't exist => add it to the list
				//newFk.add(listFK.get(i));
				entity.storeForeignKey(newForeignKey);
				changeCount++;
				_updateLogger.println(" . Foreign key '" + sFkName + "' added");
			}
		}
//		// add new fk cols to document
//		if (!newFk.isEmpty())
//		{
//			addForeignKeyParts(entity, newFk);
//		}

		// --- Number of changes
		return changeCount;
	}
	
	/**
	 * Updates an existing ENTITY in the REPOSITORY
	 */
/***
	private int updateEntity( RepositoryModel repositoryModel, DatabaseMetaData dbmd, String sCatalog,
			String sSchema, String sTableName, Entity entity) throws SQLException 
	{
		int changeCount = 0;
		Column[] columns = entity.getColumns();

		List listColumns = getColumnsMetaData(dbmd, sCatalog, sSchema, sTableName);

		//--- Get the PrimaryKey columns list
		List listPK = getColumnsPK(dbmd, sCatalog, sSchema, sTableName);

		ForeignKey[] fkeys = entity.getForeignKeys();
		//--- Get the ForeignKeys list
		List listFK = getForeignKeyParts(dbmd, sCatalog, sSchema, sTableName);

		//--- For each column in the repository ...
		for (int i = 0; i < columns.length; i++) 
		{
			Column e = columns[i];
			String sColumnName = e.getDatabaseName();
			// Does it still exist in the DATABASE ?
			if (checkColumnExistence(sColumnName, listColumns) != true) {
				// --- This column doesn't exist in the DB => remove it from the repo
				entity.removeColumn(e);
				changeCount++;
				_updateLogger.println(" . Column '" + sColumnName + "' removed");
			}
		}

		//--- For each fk in the repository ...
		for (int i = 0; i < fkeys.length; i++) 
		{
			ForeignKey e = fkeys[i];
			String sFkName = e.getName();
			// Does it still exist in the DATABASE ?
			if (!checkFkExistence(sFkName, listFK)) {
				// --- This column doesn't exist in the DB => remove it from the repo
				entity.removeForeignKey(e);
				changeCount++;
				_updateLogger.println(" . Foreign key '" + sFkName + "' removed");
			}
		}

		//--- For each column of the table in the DataBase ...
		int n = listColumns.size();
		for (int i = 0 ; i < n ; i++) 
		{
			ColumnMetaData columnMD = (ColumnMetaData) listColumns.get(i);
			String sColumnName = columnMD.getName();
			//--- Search this column in the REPOSITORY
			Column column = entity.getColumn(sColumnName);
			if ( column != null ) 
			{
				// The column exists => update it
				changeCount = changeCount + updateEntityAttribute(column, columnMD, listPK);

			} else {
				// The column doesn't exist => add it
				column = addEntityAttribute(entity, columnMD, listPK);
				_updateLogger.println(" . Column '" + sColumnName + "' added");
				changeCount++;
			}
			//--- If this column is a member of a Foreign Key
			setFkAttribute(sColumnName, column, listFK);
		}

		//--- For each FK of the table in the DataBase ...( v 0.9.0 )
		List newFk = new ArrayList();
		for (int i = 0 ; i < listFK.size() ; i++) 
		{
			ForeignKeyMetaData fkMD = (ForeignKeyMetaData) listFK.get(i);
			String sFkName = fkMD.getFkName();
			// --- Search this foreign key in the REPOSITORY
			ForeignKey foreignKey = entity.getForeignKey(sFkName);
			if ( foreignKey == null ) 
			{
				// The foreign key doesn't exist => add it to the list
				newFk.add(listFK.get(i));
				changeCount++;
				_updateLogger.println(" . Foreign key '" + sFkName + "' added");
			}
		}
		// add new fk cols to document
		if (!newFk.isEmpty())
		{
			addForeignKeyParts(entity, newFk);
		}

		// --- Number of changes
		return changeCount;
	}
***/
	
/***
	private LinkedList getColumnsMetaData(DatabaseMetaData dbmd, String sCatalog, String sSchema, String sTable)
			throws SQLException {
		LinkedList list = new LinkedList();
		// --- Get the columns of the table ...
		ResultSet rsColumns = dbmd.getColumns(sCatalog, sSchema, sTable, "%");

		// --- For each column of the table ...
		while (rsColumns.next()) {
			// --- Init colum meta-data informations
			ColumnMetaData columnMD = new ColumnMetaData(rsColumns);
			list.add(columnMD);
		}
		rsColumns.close();
		return list;
	}
***/
	
//	private boolean checkColumnExistence(String sColumnName, List listColumns) {
//		int n = listColumns.size();
//		for (int i = 0; i < n; i++) {
//			ColumnMetaData columnMD = (ColumnMetaData) listColumns.get(i);
//			String sName = columnMD.getName();
//			if (sName != null) {
//				if (sName.equalsIgnoreCase(sColumnName)) {
//					return true; // Found
//				}
//			}
//		}
//		return false; // Not found
//	}

//	/**
//	 * @param sFkName
//	 * @param listFK
//	 * @return
//	 * @since v 0.9.0
//	 */
//	private boolean checkFkExistence(String sFkName, List listFK) {
//		int n = listFK.size();
//		for (int i = 0; i < n; i++) {
//			ForeignKeyMetaData fkMD = (ForeignKeyMetaData) listFK.get(i);
//			String sName = fkMD.getFkName();
//			if (sName != null) {
//				if (sName.equalsIgnoreCase(sFkName)) {
//					return true; // Found
//				}
//			}
//		}
//		return false; // Not found
//	}

	private boolean checkTableExistsInDatabase(String sTableName, LinkedList databaseTables) {
		// --- Search the Table Name in the Database Tables
		int n = databaseTables.size();
		for (int i = 0; i < n; i++) {
			String s = (String) databaseTables.get(i);
			if (s != null) {
				if (s.equals(sTableName))
					return true;
			}
		}
		return false;
	}
}
