package org.telosys.tools.repository.config;

import java.sql.Types;
import java.util.StringTokenizer;

import org.telosys.tools.commons.jdbctypes.JdbcTypes;
import org.telosys.tools.commons.jdbctypes.JdbcTypesManager;
import org.telosys.tools.repository.model.Column;

/**
 * @author L.GUERIN
 * 
 */
public class DefaultInitializerChecker implements InitializerChecker
{   
	
	//------------------------------------------------------------------------------
    public DefaultInitializerChecker() {
		super();
	}

	//------------------------------------------------------------------------------
    /**
     * Transform the given token with only the first char in Upper Case  
     * ie : "aBcDeFg" --> "Abcdefg"
     * @param s 
     * @return transformed string
     */
    private String transformToken(String s)
    {
        return s.substring(0, 1).toUpperCase() + s.substring(1).toLowerCase();
    }

    //------------------------------------------------------------------------------
    /**
     * Transform the given string to "CamelCase", using "_" as word separator  
     * ie : "ORDER_ITEM" --> "OrderItem"
     * @param sName
     * @return transformed string
     */
    private String toCamelCase(String sName)
    {
        if (sName != null)
        {
            StringBuffer sb = new StringBuffer( sName.length() );
            String sToken = null;
            String s = sName.trim(); // to be secure
            StringTokenizer st = new StringTokenizer(s, "_");
            while (st.hasMoreTokens())
            {
                sToken = st.nextToken();
                //sClassName = sClassName + transformToken( sToken ) ;
                sb.append( transformToken( sToken ) );
            }
            //return sClassName;
            return sb.toString();
        }
        else
        {
            return null;
        }
    }

	
//    //--- Conversion table : JDBC Type Code (cf java.sql.Types) to Java Type
//    public static HashMap htSqlTypeToJavaType = new HashMap() ; 
//    static
//    {
//        //--- INTEGER TYPES
//        htSqlTypeToJavaType.put(new Integer(Types.TINYINT),  "int"); 
//        htSqlTypeToJavaType.put(new Integer(Types.SMALLINT), "int");
//        htSqlTypeToJavaType.put(new Integer(Types.INTEGER),  "int");
//        htSqlTypeToJavaType.put(new Integer(Types.BIGINT),   "long");
//        
//        //--- FLOAT TYPES
//        htSqlTypeToJavaType.put(new Integer(Types.REAL),    "double");
//        htSqlTypeToJavaType.put(new Integer(Types.FLOAT),   "double");
//        htSqlTypeToJavaType.put(new Integer(Types.DOUBLE),  "double");
//        
//        htSqlTypeToJavaType.put(new Integer(Types.DECIMAL), "double"); 
//        htSqlTypeToJavaType.put(new Integer(Types.NUMERIC), "double"); 
//
//        //--- BOOLEAN TYPES
//        htSqlTypeToJavaType.put(new Integer(Types.BIT),     "boolean"); 
//        
//        //--- STRING TYPES
//        htSqlTypeToJavaType.put(new Integer(Types.CHAR),        "String");
//        htSqlTypeToJavaType.put(new Integer(Types.VARCHAR),     "String");
//        htSqlTypeToJavaType.put(new Integer(Types.LONGVARCHAR), "String");
//
//        //--- DATE & TIME TYPES
//        htSqlTypeToJavaType.put(new Integer(Types.DATE),      "Date");
//        htSqlTypeToJavaType.put(new Integer(Types.TIMESTAMP), "Date");
//        htSqlTypeToJavaType.put(new Integer(Types.TIME),      "Date");
//
//    }
    
//    //------------------------------------------------------------------------------
//    // Check if the first char of the String is an upper case
//    //------------------------------------------------------------------------------
//    private boolean isFirstCharUpperCase(String s)
//    {
//        if (s.length() < 1) return false ;
//
//        if (s.substring(0, 1).toUpperCase().equals(s.substring(0, 1)))
//        {
//            return true;
//        }
//        else
//        {
//            return false;
//        }
//    }

//    //------------------------------------------------------------------------------
//    // Informations about the current class
//    //------------------------------------------------------------------------------
//    public String about()
//    {
//        return ("Standard InitializerChecker class (provided by plugin)");
//    }

//    //------------------------------------------------------------------------------
//    // Table name conform to the Java Naming Conventions
//    //------------------------------------------------------------------------------
//    private String getValidTableName(String sTableName)
//    {
//        return (sTableName.substring(0, 1).toUpperCase() + sTableName.substring(1, sTableName.length()).toLowerCase());
//
//    }

    //------------------------------------------------------------------------------
    // Classes
    //------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.telosys.daogen.common.InitializerChecker#getClassNameForVO(java.lang.String)
     */
    public String getJavaBeanClassName(String sTableName)
    {
    	return toCamelCase(sTableName);
    	// First char Upper Case 
        //return (sTableName.substring(0, 1).toUpperCase() + sTableName.substring(1, sTableName.length()).toLowerCase());
        //return getValidTableName(sTableName) + "VO";
    }

    //------------------------------------------------------------------------------
    // Value Object Attribute Name from Database column name and type
    //------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.telosys.daogen.common.InitializerChecker#getAttributeName(java.lang.String, java.lang.String)
     */
    public String getAttributeName(String sColumnName, String sColumnTypeName, int iColumnTypeCode)
    {
        //--- Colum name converted in "CamelCase"
        String s = toCamelCase(sColumnName);
        
        //--- Java attribute => Force the first char to LowerCase
        return s.substring(0,1).toLowerCase() + s.substring(1,s.length()) ;
    }

    //------------------------------------------------------------------------------
    // Value Object Attribute Type from Database column type
    //------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.telosys.daogen.common.InitializerChecker#getAttributeType(java.lang.String)
     */
    public String getAttributeType(String sColumnTypeName, int iJdbcTypeCode, boolean bColumnNotNull)
    {
    	//--- Special cases for Date/Time/Timestamp
    	if ( iJdbcTypeCode == Types.DATE  || iJdbcTypeCode == Types.TIME || iJdbcTypeCode == Types.TIMESTAMP ) 
    	{
    		// Force "java.util.Date" : see also getAttributeDateType() for DATE_ONLY, TIME_ONLY, etc...
    		return "java.util.Date";
    	}
    	JdbcTypes types = JdbcTypesManager.getJdbcTypes();
    	String sJavaType = types.getJavaTypeForCode(iJdbcTypeCode, bColumnNotNull);
    	return ( sJavaType != null ? sJavaType : "java.lang.String" );    	
    }

    //------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.telosys.daogen.common.InitializerChecker#getAttributeType(java.lang.String)
     */
    public String getAttributeLongTextFlag (String sColumnType, int iColumnTypeCode, String sJavaType )
    {
    	if ( sJavaType.equals("java.lang.String") )
    	{
    		if (   iColumnTypeCode == Types.LONGVARCHAR
    			|| iColumnTypeCode == Types.CLOB
    			|| iColumnTypeCode == Types.BLOB )
    		{
    			// Considered as a "Long Text"
    			//return ConstDbRep.SPECIAL_LONG_TEXT_TRUE ; 
    			return Column.SPECIAL_LONG_TEXT_TRUE ; 
    		}
    	}
    	return null ;
    }
    
    public String getAttributeDateType(String sColumnType, int iColumnTypeCode, String sJavaType )
    {
    	if ( sJavaType.equals("java.util.Date") )
    	{
        	switch ( iColumnTypeCode )
        	{
        		//--- Type of Date :
        		case Types.DATE : 
//        			return ConstDbRep.SPECIAL_DATE_ONLY ; 
        			return Column.SPECIAL_DATE_ONLY ; 
        		case Types.TIME : 
//        			return ConstDbRep.SPECIAL_TIME_ONLY ; 
        			return Column.SPECIAL_TIME_ONLY ; 
        		case Types.TIMESTAMP : 
//        			return ConstDbRep.SPECIAL_DATE_AND_TIME ;
        			return Column.SPECIAL_DATE_AND_TIME ;
        	}
    	}
    	return null ;
    }
    
//    /* (non-Javadoc)
//     * @see org.telosys.daogen.common.InitializerChecker#getClassNameForVOList(java.lang.String)
//     */
//    public String getClassNameForVOList(String sTableName)
//    {
//        return getValidTableName(sTableName) + "VOList";
//    }

//    /* (non-Javadoc)
//     * @see org.telosys.daogen.common.InitializerChecker#getClassNameForDAO(java.lang.String)
//     */
//    public String getClassNameForDAO(String sTableName)
//    {
//        return getValidTableName(sTableName) + "DAO";
//    }

//    /* (non-Javadoc)
//     * @see org.telosys.daogen.common.InitializerChecker#getClassNameForTestDAO(java.lang.String)
//     */
//    public String getClassNameForTestDAO(String sTableName)
//    {
//        return "Test" + getValidTableName(sTableName) + "DAO";
//    }
//
//    /* (non-Javadoc)
//     * @see org.telosys.daogen.common.InitializerChecker#getClassNameForTestDAOList(java.lang.String)
//     */
//    public String getClassNameForTestDAOList(String sTableName)
//    {
//        return "Test" + getValidTableName(sTableName) + "DAOList";
//    }

    //------------------------------------------------------------------------------
    // Value Object Attributes
    //------------------------------------------------------------------------------
//    /**
//     * Mthode qui gre le retour utilisateur au niveau du contrle du nom Java
//     * de l'atribut
//     */
//    public boolean checkAttributeName(String sBaseName, String sJavaName)
//    {
//    	if ((sBaseName != null) && (sJavaName != null) && (isFirstCharUpperCase(sJavaName)))
//    	{
//    		return sBaseName.equalsIgnoreCase(sJavaName);
//    	}
//    	return false;
//    }
//    
//    
//    public boolean checkAttributeType(int sJDBCType, String sJavaType)
//    {
//    	String newJdbcType = getAttributeType( "", sJDBCType);
//    	
//    	if ( newJdbcType == null )
//    		return false;
//    	
//    	return ( newJdbcType.equalsIgnoreCase(sJavaType) );
//    }
 
    //------------------------------------------------------------------------------
    // Classes names
    //------------------------------------------------------------------------------

//    public boolean checkClassNameForVO(String sName)
//    {
//        return isFirstCharUpperCase(sName);
//    }
//
//    public boolean checkClassNameForVOList(String sName)
//    {
//        return isFirstCharUpperCase(sName);
//    }
//
//    public boolean checkClassNameForDAO(String sName)
//    {
//        return isFirstCharUpperCase(sName);
//    }
//
//    public boolean checkClassNameForTestDAO(String sName)
//    {
//        return isFirstCharUpperCase(sName);
//    }
//
//    public boolean checkClassNameForTestDAOList(String sName)
//    {
//        return isFirstCharUpperCase(sName);
//    }

//    /*
//     * (non-Javadoc)
//     * 
//     * @see org.telosys.daogen.common.InitializerChecker#getFieldTypes()
//     */
//    public String[] getFieldTypes()
//    {
//        return new String[] { "String", "int", "long", "double", "Date", "boolean" };
//    }

//    public boolean storeBoolean()
//    {
//        return true; // Boolean strored by default
//    }
}