package org.telosys.tools.repository.persistence;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.telosys.tools.commons.TelosysToolsException;
import org.telosys.tools.commons.TelosysToolsLogger;
import org.telosys.tools.repository.model.RepositoryModel;
import org.telosys.tools.repository.persistence.util.CommandException;

public class StandardFilePersistenceManager extends GenericPersistenceManager
{

	private String _fileName = null ;
	
	public StandardFilePersistenceManager(String fileName, TelosysToolsLogger logger) {
		super(logger);
		this._fileName = fileName ;
	}

	public StandardFilePersistenceManager(File file, TelosysToolsLogger logger) {
		super(logger);
		this._fileName = file.getAbsolutePath() ;
	}

	public RepositoryModel load() throws TelosysToolsException {
		try {
			InputStream is = new FileInputStream(this._fileName);
			RepositoryModel repositoryModel =  super.load(is);
			try {
				is.close();
			} catch (IOException e) {
				throw new TelosysToolsException("cannot close file " + this._fileName, e);
			}
			return repositoryModel ;
		} catch (FileNotFoundException e) {
			throw new TelosysToolsException("file not found : " + this._fileName, e);
		} catch (CommandException e) {
			throw new TelosysToolsException("parsing exception : " + this._fileName, e);
		}
		
	}

	public void save(RepositoryModel model) throws TelosysToolsException {
		try {
			OutputStream os = new FileOutputStream(this._fileName, false);
			super.save(os, model);
			try {
				os.close();
			} catch (IOException e) {
				throw new TelosysToolsException("cannot close file " + this._fileName, e);
			}
		} catch (FileNotFoundException e) {
			throw new TelosysToolsException("file not found : " + this._fileName, e);
		}
	
	}

}
