package org.telosys.tools.repository.persistence.commande;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.telosys.tools.repository.persistence.util.CommandException;
import org.telosys.tools.repository.persistence.util.ProcessContext;

public class CommandManager implements ICommandManager {

	private static ICommandManager manager;
	static {
		// Instanciation parser
		List<AbstractCommand> cmds = new ArrayList<AbstractCommand>();
		
		cmds.add(new CommandElementColumn());
		
		cmds.add(new CommandElementForeignKey());
		cmds.add(new CommandElementForeignKeyColumn());
		
		cmds.add(new CommandElementGeneratedValue());

		cmds.add(new CommandElementLink());
		
//		cmds.add(new CommandElementJoinFK());
		cmds.add(new CommandElementJoinTable());
		cmds.add(new CommandElementJoinColumns() );
		cmds.add(new CommandElementInverseJoinColumns() );
		cmds.add(new CommandElementJoinColumn() );
		
		cmds.add(new CommandElementSequenceGenerator());
		cmds.add(new CommandElementTable());
		cmds.add(new CommandElementTableGenerator());
		cmds.add(new CommandNode());
		cmds.add(new CommandNodeList());
		manager = new CommandManager(cmds);
	}	
	
	public static ICommandManager getCommandManager() {
		return manager;
	}
	
	private final List liste;
	
	private CommandManager(List liste) {
		super();
		this.liste = liste;
	}

	public ICommandContext searchCommand(final ProcessContext processContext) throws CommandException {
		ICommandContext iCommandContextFound = null;
		for (Iterator iterator = liste.iterator(); iterator.hasNext();) {
			ICommandContext iCommandContext = (ICommandContext) iterator.next();
			if (iCommandContext.accept(processContext)) {
				iCommandContextFound = iCommandContext;
				break;
			}
		}
		
		if (iCommandContextFound != null) {
			return iCommandContextFound;
		} else {
			throw new CommandException();
		}
	}


	public List getListe() {
		return liste;
	}

}
